/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#include "error.h"
#include "config.h"

#include <cstdio>
#include <cstdlib>
#include <cstdarg>
#include <exception>

static void report(const char* prefix, const char* file,
           const char* func, long line, const char* fmt,
           va_list vl)
{
    static char msg[4096];

    std::vsnprintf(msg, sizeof(msg), fmt, vl);
    std::fprintf(stderr, " [%s in %s:%ld] %s%s\n", file, func,
        line, prefix, msg );
}


void linktest_fatal(const char* file, const char* func, long line, const char* fmt, ...)
{
    va_list vl;

    va_start(vl, fmt);
    report("fatal: ", file, func, line, fmt, vl);
    va_end(vl);

    std::fflush(NULL);
    std::terminate();
}

void linktest_error(const char* file, const char* func, long line, const char* fmt, ...)
{
    va_list vl;

    va_start(vl, fmt);
    report("error: ", file, func, line, fmt, vl);
    va_end(vl);

    std::fflush(NULL);
}

void linktest_warn(const char* file, const char* func, long line, const char* fmt, ...)
{
    va_list vl;

    va_start(vl, fmt);
    report("warning: ", file, func, line, fmt, vl);
    va_end(vl);

    std::fflush(NULL);
}

void linktest_info(const char* file, const char* func, long line, const char* fmt, ...)
{
    va_list vl;

    va_start(vl, fmt);
    report("info: ", file, func, line, fmt, vl);
    va_end(vl);
}

void linktest_debug(const char* file, const char* func, long line, const char* fmt, ...)
{
    va_list vl;

    va_start(vl, fmt);
    report("debug: ", file, func, line, fmt, vl);
    va_end(vl);

    std::fflush(NULL);
}