/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#ifndef LINKTEST_GPU_NVIDIA_H
#define LINKTEST_GPU_NVIDIA_H

#if 1 != HAVE_CUDA
#error gpu_nvidia can only compile with HAVE_CUDA=1
#endif

#include "config.h"
#include <functional>
#include <cuda.h>

namespace linktest
{
namespace cuda
{

// A GPU device handle.
class GpuDevice
{
public:
    GpuDevice(int number);
    CUdevice        cuda_handle();
private:
    int             number_;
    CUdevice        cuda_dev_;

};

/* A GPU context object
 * Note: `GpuContext` is currently a singleton. This may change in the future
 * but at the moment only a single `GpuContext` per process (and consequently
 * only a single device per process) is supported. See also #7.
 */
class GpuContext
{

public:
    GpuContext(GpuDevice* dev);
    ~GpuContext();
    static GpuContext* singleton();
    GpuDevice*         device();
    void*              memalloc(uint64_t len);
    int                memfree(void* p);

    /* Execute the function `f` with `cuda_ctx_` on the stack. The context
     * is again removed from the stack after the execution of the function.
     * The return value of `f` is returned.
     */
    int                execute(std::function<int(void)> f);
private:
    GpuDevice*         dev_;
    CUcontext          cuda_ctx_;
};

/* Report an error from `callee` via the `error()` function. The function
 * does not alter the code path, i.e., it does not throw an exception.
 */
void report_cuda_error(const char* callee, CUresult err);


}
}

#endif
